@@ -8,6 +8,7 @@ use File::Spec;
 use Sys::Hostname;
 use DBI;
 use Helios::ObjectDriver::DBI;
+require XML::Simple;
 
 use Helios::Error;
 use Helios::Job;
@@ -15,11 +16,8 @@ use Helios::Config;
 use Helios::ConfigParam;
 use Helios::LogEntry;
 use Helios::LogEntry::Levels qw(:all);
-# [LH] [2013-10-04]: Using Helios::JobType instead of TheSchwartz::FuncMap now.
-use Helios::JobType;
-use Helios::Error::JobTypeError;
 
-our $VERSION = '2.72_0950';
+our $VERSION = '2.601_3610';
 
 # FILE CHANGE HISTORY:
 # [2011-12-07]: Updated to support new Helios::Logger API.  Added 
@@ -104,26 +102,7 @@ our $VERSION = '2.72_0950';
 # initialization errors.  [RT79690]
 # [LH] [2013-08-19]: Removed old commented out code and clarified comments on 
 # job initialization error handling.
-# [LH] [2013-10-04]: Added code to start conversion to new Helios class 
-# structure and support virtual jobtypes feature.  New new() constructor 
-# initializes attribute hashref values and can only be called as a class 
-# method.  Added set/get/addJobType(), set/getAltJobTypes(), 
-# set/getAltJobtypeids(), addAltJobtypeid(), lookupJobtypeid(), 
-# lookupAltJobtypeids().  Switched all code that used TheSchwartz::FuncMap to 
-# use Helios::JobType.  Replaced set/getFuncid() with new version that mirrors 
-# set/getJobType() (set/getFuncid() will be deprecated on final release).  
-# Replaced jobsWaiting() with new version that uses Helios::JobType and scans 
-# for all jobtypes (primary and alternates) if alternate jobtypes are set.
-# [LH] [2013-10-04]: Removed 'require XML::Simple' line because Helios::Service 
-# has not used that in a long time.
-# [LH] [2013-10-18]: Added grab_for() and JobLockInterval() to implement new 
-# retry API.  Added $CACHED_HOSTNAME and modified prep() to reduce calls to 
-# Sys::Hostname::hostname(). 
-# [LH] [2013-10-24]: Removed old, already commented out code.  Added POD for 
-# new methods added in 2.7x development series.  Marked getFuncidFromDb() as
-# deprecated; its function has been replaced by lookupJobtypeid().
-# [LH] [2014-02-28]: Changed max_retries() and retry_delay() to default to zero
-# rather than undef to eliminate some warnings.
+
 
 =head1 NAME
 
@@ -189,23 +168,14 @@ the service is not in OVERDRIVE mode, the worker process will exit.
 our $CACHED_CONFIG;
 our $CACHED_CONFIG_RETRIEVAL_COUNT = 0;
 our $WORKER_START_TIME = 0;
-# [LH] [2013-10-18]: Added $CACHED_HOSTNAME and modified prep() to reduce calls to 
-# Sys::Hostname::hostname().  
-our $CACHED_HOSTNAME = '';
 
 our %INIT_LOG_CLASSES;	# for the logging system
 our $INIT_CONFIG_CLASS; # for config system
 
 our $DRIVER;	# for caching the Data::ObjectDriver
 
-sub max_retries { $_[0]->MaxRetries() || 0; }
-sub retry_delay { $_[0]->RetryInterval() || 0; }
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-# [LH] [2013-10-18]: Added grab_for() and JobLockInterval() to implement new 
-# retry API.  Like TheSchwartz's setup, the JobLockInterval() defaults to 
-# 3600 sec (1 hr).
-sub grab_for { $_[0]->JobLockInterval() || 3600 }
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
+sub max_retries { $_[0]->MaxRetries(); }
+sub retry_delay { $_[0]->RetryInterval(); }
 
 sub work {
 	my $class = shift;
@@ -437,9 +407,6 @@ These accessors will be needed by subclasses of Helios::Service.
  get/setIniFile()
  get/setJob()
  get/setJobType()
- get/setAltJobTypes(), addAltJobType()
- get/setJobTypeid()
- get/setAltJobtypeids(), addAltJobtypeid()
  errstr()
  debug()
 
@@ -462,52 +429,11 @@ sub getJob { return $_[0]->{job}; }
 sub setJobType { $_[0]->{jobType} = $_[1]; }
 sub getJobType { return $_[0]->{jobType}; }
 
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-# [LH] [2013-10-04] Virtual jobtype code.
-sub setAltJobTypes {
-	my $self = shift;
-	$self->{altJobTypes} = [@_];
-}
-sub getAltJobTypes {
-	if ( defined $_[0]->{altJobTypes} ) {
-		return @{ $_[0]->{altJobTypes} };
-	} else {
-		return undef;
-	}
-}
-sub addAltJobType {
-	push(@{ $_[0]->{altJobTypes} }, $_[1]);
-}
-
-# [LH] [2013-10-04] Virtual jobtype code.
-sub setJobtypeid { $_[0]->{jobtypeid} = $_[1]; }
-sub getJobtypeid { return $_[0]->{jobtypeid}; }
-
-sub setAltJobtypeids {
-	my $self = shift;
-	$self->{altJobtypeids} = [@_];
-}
-sub getAltJobtypeids {
-	if ( defined $_[0]->{altJobtypeids} ) {
-		return @{ $_[0]->{altJobtypeids} };
-	} else {
-		return undef;
-	}
-}
-sub addAltJobtypeid {
-	push(@{ $_[0]->{altJobtypeids} }, $_[1]);
-}
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
-
-
 sub setConfig { $_[0]->{config} = $_[1]; }
 sub getConfig { return $_[0]->{config}; }
 
-# [LH] [2013-10-04]: Virtual jobtypes.  Changed set/getFuncid() for 
-# compatibility with set/getJobtypeid().  Set/getFuncid() is DEPRECATED;
-# retained for now for backward compatibility with Helios 2.6x and earlier.
-sub setFuncid { $_[0]->{jobtypeid} = $_[1]; }
-sub getFuncid { return $_[0]->{jobtypeid}; }
+sub setFuncid { $_[0]->{funcid} = $_[1]; }
+sub getFuncid { return $_[0]->{funcid}; }
 
 sub setIniFile { $_[0]->{inifile} = $_[1]; }
 sub getIniFile { return $_[0]->{inifile}; }
@@ -528,44 +454,31 @@ sub getDriver {
 sub errstr { my $self = shift; @_ ? $self->{errstr} = shift : $self->{errstr}; }
 sub debug { my $self = shift; @_ ? $self->{debug} = shift : $self->{debug}; }
 
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-# [LH] [2013-10-04] New constructor initializes attributes in the underlying
-# object structure and can only be called as a class method.  
 
 =head1 CONSTRUCTOR
 
 =head2 new()
 
-The new() method creates a new service class instance.  It initializes all of 
-the underlying attribute values and sets the instance's jobType to the name of 
-the class.
+The new() method doesn't really do much except create an object of the appropriate class.  (It can 
+overridden, of course.)
+
+It does set the job type for the object (available via the getJobType() method).
 
 =cut
 
 sub new {
-	my $cl = shift;
-	my $self = {
-		'jobType'       => undef,
-		'altJobTypes'   => undef,
-		'jobtypeid'     => undef,
-		'altJobtypeids' => undef,
-		'hostname'      => undef,
-		'inifile'       => undef,
-		'job'           => undef,
-		
-		'config' => undef,
-		'debug'  => undef,
-		'errstr' => undef,
-	};
-	bless $self, $cl;
+	my $caller = shift;
+	my $class = ref($caller) || $caller;
+#	my $self = $class->SUPER::new(@_);
+	my $self = {};
+	bless $self, $class;
 
 	# init fields
-	my $jobtype = $cl;
+	my $jobtype = $caller;
 	$self->setJobType($jobtype);
 
 	return $self;
 }
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
 
 
 =head1 INTERNAL SERVICE CLASS METHODS
@@ -631,22 +544,9 @@ sub prep {
 	}
 
 	# pull other parameters from environment
-
-# END CODE Copyright (C) 2012 by Andrew Johnson.
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-	# If hostname value is not set,
-	# 1) use the cached value if we have one, or 
-	# 2) go ahead and call hostname() (and cache it for later)
-	if ( !defined($self->getHostname()) ) {
-	# [LH] [2013-10-18] Changed hostname handling to reduce hostname lookups.
-		if ( $CACHED_HOSTNAME ) {
-			$self->setHostname($CACHED_HOSTNAME);
-		} else {
-			$CACHED_HOSTNAME = hostname();
-			$self->setHostname($CACHED_HOSTNAME);
-		}
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.		
-# BEGIN CODE Copyright (C) 2012 by Andrew Johnson.
+	# only bother to set hostname if it isn't already set	
+	unless ( defined($self->getHostname()) ) {
+		$self->setHostname(hostname);
 	}
 	
 	if ( defined($ENV{HELIOS_DEBUG}) ) {
@@ -762,7 +662,7 @@ sub getConfigFromDb {
 }
 
 
-=head2 getFuncidFromDb() [DEPRECATED]
+=head2 getFuncidFromDb()
 
 Queries the collective database for the funcid of the service class and 
 returns it to the calling routine.  The service name used in the query is the 
@@ -773,9 +673,6 @@ with a particular service class, so it can scan the job table for waiting jobs.
 If their are jobs for the service waiting, helios.pl may launch new worker 
 processes to perform these jobs.
 
-As of Helios 2.80, getFuncidFromDb() has been replaced by lookupJobtypeid().  
-This method is thus deprecated.
-
 =cut
 
 sub getFuncidFromDb {
@@ -803,57 +700,6 @@ sub getFuncidFromDb {
 }
 
 
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-# [LH] [2013-10-04] Virtual jobtype code.
-=head2 lookupAltJobtypeids(@jobtypenames)
-
-The lookupAltJobtypeids() method uses the lookupJobtypeid() method to determine 
-the jobtypeids of all of the service instance's alternate jobtypes.  If given 
-a list of jobtype names, these will override any jobtypes previously set with
-the setAltJobTypes() or addAltJobType() methods.
-
-Usually, "alternate" jobtypes and jobtypes specified on the helios.pl 
-command line using the --jobtypes option.  The "primary" jobtype is the jobtype
-matching the service class's name.
-
-=cut
-
-sub lookupAltJobtypeids {
-	my $self = shift;
-	my @jobtypes = @_ || $self->getAltJobTypes();
-	my $config = $self->getConfig();
-	my @ids;
-	
-	for (@jobtypes) {
-		my $jtid = $self->lookupJobtypeid($_);
-		unless ($jtid) { Helios::Error::JobTypeError->throw("lookupAltJobtypeids(): $_ cannot be found in collective database."); }
-		push(@ids, $jtid);
-		$self->addAltJobtypeid($jtid);
-	}
-	return @ids;
-}
-
-
-=head2 lookupJobtypeid($jobtypename)
-
-Given the name of a jobtype, lookupJobtypeid() uses the Helios::JobType class 
-to find the jobtypeid of the jobtype and returns it to the calling routine.  
-If the jobtype does not exist, the method returns undef.
-
-=cut
-
-sub lookupJobtypeid {
-	my $self = shift;
-	my $jt = shift;
-
-	my $jobtype = Helios::JobType->lookup(name => $jt, config => $self->getConfig());
-	if ($jobtype) {
-		return $jobtype->getJobtypeid();
-	} else {
-		return undef;
-	}
-}
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
 
 
 =head2 jobsWaiting() 
@@ -863,43 +709,26 @@ waiting.  Only meant for use with the helios.pl service daemon.
 
 =cut
 
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-# [LH] [2013-10-04] jobsWaiting() replaced with new version for virtual 
-# jobtypes.    
 sub jobsWaiting {
 	my $self = shift;
-	my $num_of_jobs = 0;
-	my $primary_jobtypeid = $self->getJobtypeid();
-	my @alt_jobtypeids;
-	my $sth;
+	my $params = $self->getConfig();
+	my $jobType = $self->getJobType();
+
+
+# BEGIN CODE Copyright (C) 2012 by Logical Helion, LLC.
+	my $jobsWaiting;
+	my $funcid = $self->getFuncid();
 	eval {
 		my $dbh = $self->dbConnect();
-		unless ( defined($primary_jobtypeid) ) {
-			$primary_jobtypeid = $self->lookupJobtypeid($self->getJobType);
-			$self->setJobtypeid($primary_jobtypeid);
-		}
-		if ( $self->getAltJobTypes() ) {
-			if ( $self->getAltJobtypeids() ) {
-				@alt_jobtypeids = $self->getAltJobtypeids();
-			} else {
-				@alt_jobtypeids = $self->lookupAltJobtypeids();
-			}
+		unless ( defined($funcid) ) {
+			$funcid = $self->getFuncidFromDb();
 		}
 		
-		if (@alt_jobtypeids) {
-			my @plhrs = ('?');	# one for the primary
-			for (@alt_jobtypeids) { push(@plhrs,'?'); }
-			my $plhrs_str = join(',' => @plhrs);
-
-			$sth = $dbh->prepare_cached("SELECT COUNT(*) FROM job WHERE funcid IN($plhrs_str) AND (run_after < ?) AND (grabbed_until < ?)");
-			$sth->execute($primary_jobtypeid, @alt_jobtypeids, time(), time());
-		} else {
-			$sth = $dbh->prepare_cached('SELECT COUNT(*) FROM job WHERE funcid = ? AND (run_after < ?) AND (grabbed_until < ?)');
-			$sth->execute($primary_jobtypeid, time(), time());
-		}
+		my $sth = $dbh->prepare_cached('SELECT COUNT(*) FROM job WHERE funcid = ? AND (run_after < ?) AND (grabbed_until < ?)');
+		$sth->execute($funcid, time(), time());
 		my $r = $sth->fetchrow_arrayref();
 		$sth->finish();
-		$num_of_jobs = $r->[0];
+		$jobsWaiting = $r->[0];
 		
 		1;
 	} or do {
@@ -907,9 +736,13 @@ sub jobsWaiting {
 		Helios::Error::DatabaseError->throw("$E");
 	};
 	
-	return $num_of_jobs;
+	return $jobsWaiting;
+# END CODE Copyright (C) 2012 by Logical Helion, LLC.
+
+
+
+
 }
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
 
 
 # BEGIN CODE Copyright (C) 2012 by Andrew Johnson.
@@ -1470,60 +1303,16 @@ sub run {
     throw Helios::Error::FatalNoRetry($_[0]->getJobType.': run() method not implemented!'); 
 }
 
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
+=head2 MaxRetries() and RetryInterval()
 
-=head2 MaxRetries(), RetryInterval(), and JobLockInterval()
-
-The MaxRetries(), RetryInterval(), and JobLockInterval() methods specify to 
-Helios the number of reattempts it should make at running a job and the 
-frequency of those attempts.   
-
-control how many times a job should be retried if it fails and 
+These methods control how many times a job should be retried if it fails and 
 how long the system should wait before a retry is attempted.  If you don't 
-defined these, jobs will not be retried if they fail.
-
-MaxRetries() is straightforward; set it to the number of times we want a job 
-to be retried if it fails.
-
-RetryInterval() is the amount of time (in seconds) to wait after a job fails 
-before a job is available to try again.
-
-JobLockInterval() is the amount of time (in seconds) a job is locked for 
-processing.  This amount of time should be enough time to make sure a job can 
-be completed or at marked as failed.  The default is 3600 sec (1 hour).  
-
-RetryInterval() and JobLockInterval() can interact in an odd way:  for example,
-if you want to retry a job every 60 secs, you can add:
-
- sub RetryInterval { 60 }
-
-to your service class.  However, your jobs will still be locked for an hour, 
-because 3600 is the JobLockInterval() default.  If you want to retry jobs 
-more frequently than a hour, you need to add a JobLockInterval() method to 
-your service class as well as a RetryInterval() method.  So, to retry jobs 
-every 60 secs, add both of the following methods to your service class:
-
- sub RetryInterval { 60 }
- sub JobLockInterval { 60 }
-
-Keep in mind this will reduce the amount of time available for your service to 
-mark a job as completed or failed.  If it has not done so by the time the 
-JobLockInterval() value has expired, the job will be seen by the Helios system 
-as available for processing again, and another worker process will pick up and 
-attempt to run the job.  So always make sure your JobLockInterval() allows 
-enough time to actually complete a job.  Another rule of thumb is to set 
-RetryInterval() and JobLockInterval() to the same value if RetryInterval() is 
-less than 3600.
+defined these, jobs will not be retried if they fail.   
 
 =cut
 
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
-
 sub MaxRetries { return undef; }
 sub RetryInterval { return undef; }
-# BEGIN CODE Copyright (C) 2013 by Logical Helion, LLC.
-sub JobLockInterval { undef }
-# END CODE Copyright (C) 2013 by Logical Helion, LLC.
 
 =head2 JobClass()
 
@@ -1598,6 +1387,8 @@ sub _require_module {
 # END CODE Copyright (C) 2012 by Logical Helion, LLC.
 
 
+
+
 1;
 __END__
 
@@ -1605,7 +1396,7 @@ __END__
 =head1 SEE ALSO
 
 L<Helios>, L<helios.pl>, L<Helios::Job>, L<Helios::Error>, L<Helios::Config>, 
-L<Helios::JobType>
+L<TheSchwartz>
 
 =head1 AUTHOR
 
